@@ -8,6 +8,7 @@ import (
 	"path/filepath"
 	"slices"
 	"strings"
+	"time"
 
 	"github.com/go-task/task/v3/errors"
 	"github.com/go-task/task/v3/internal/filepathext"
@@ -16,9 +17,6 @@ import (
 )
 
 var (
-	// ErrIncludedTaskfilesCantHaveDotenvs is returned when a included Taskfile contains dotenvs
-	ErrIncludedTaskfilesCantHaveDotenvs = errors.New("task: Included Taskfiles can't have dotenv declarations. Please, move the dotenv declaration to the main Taskfile")
-
 	defaultTaskfiles = []string{
 		"Taskfile.yml",
 		"taskfile.yml",
@@ -29,7 +27,6 @@ var (
 		"Taskfile.dist.yaml",
 		"taskfile.dist.yaml",
 	}
-
 	allowedContentTypes = []string{
 		"text/plain",
 		"text/yaml",
@@ -44,7 +41,7 @@ var (
 // at the given URL with any of the default Taskfile files names. If any of
 // these match a file, the first matching path will be returned. If no files are
 // found, an error will be returned.
-func RemoteExists(ctx context.Context, l *logger.Logger, u *url.URL) (*url.URL, error) {
+func RemoteExists(ctx context.Context, l *logger.Logger, u *url.URL, timeout time.Duration) (*url.URL, error) {
 	// Create a new HEAD request for the given URL to check if the resource exists
 	req, err := http.NewRequest("HEAD", u.String(), nil)
 	if err != nil {
@@ -54,6 +51,9 @@ func RemoteExists(ctx context.Context, l *logger.Logger, u *url.URL) (*url.URL,
 	// Request the given URL
 	resp, err := http.DefaultClient.Do(req.WithContext(ctx))
 	if err != nil {
+		if errors.Is(ctx.Err(), context.DeadlineExceeded) {
+			return nil, &errors.TaskfileNetworkTimeoutError{URI: u.String(), Timeout: timeout}
+		}
 		return nil, errors.TaskfileFetchFailedError{URI: u.String()}
 	}
 	defer resp.Body.Close()
