@@ -7,15 +7,15 @@ import remarkGfm from 'remark-gfm';
 
 import { DISCORD_URL } from './constants';
 import { GITHUB_URL } from './constants';
+import { BLUESKY_URL } from './constants';
 import { MASTODON_URL } from './constants';
 import { TWITTER_URL } from './constants';
+import { STACK_OVERFLOW } from './constants';
+import { ANSWER_OVERFLOW } from './constants';
 
 import lightCodeTheme from './src/themes/prismLight';
 import darkCodeTheme from './src/themes/prismDark';
 
-import { getTranslationProgress } from './src/api/crowdin.js';
-const translationProgress = getTranslationProgress();
-
 const config: Config = {
   title: 'Task',
   tagline: 'A task runner / simpler Make alternative written in Go ',
@@ -62,7 +62,10 @@ const config: Config = {
             }
           }
         },
-        blog: {},
+        blog: {
+          blogSidebarTitle: 'All posts',
+          blogSidebarCount: 'ALL'
+        },
         theme: {
           customCss: [
             './src/css/custom.css',
@@ -177,6 +180,11 @@ const config: Config = {
             {
               label: 'Twitter',
               href: TWITTER_URL
+            },
+						{
+              label: 'Bluesky',
+              href: BLUESKY_URL,
+              rel: 'me'
             },
             {
               label: 'Mastodon',
@@ -187,6 +195,14 @@ const config: Config = {
               label: 'Discord',
               href: DISCORD_URL
             },
+            {
+              label: 'Stack Overflow',
+              href: STACK_OVERFLOW
+            },
+            {
+              label: 'Answer Overflow',
+              href: ANSWER_OVERFLOW
+            },
             {
               label: 'OpenCollective',
               href: 'https://opencollective.com/task'
